<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations;

use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Exceptions\InvalidLicenseException;

class DemoValidation implements Validation
{
    /**
     * Demo license type value
     */
    public const DEMO_LICENSE_TYPE = 'Demo';

    /**
     * Demo Token payload keys
     */
    public const PK_LIFETIME_UNIT = 'lifetime_unit';
    public const PK_LIFETIME_DURATION = 'lifetime_duration';
    public const PK_EXPIRY_DATE = 'expiry_date';

    /**
     * @var array - Decoded License Token
     */
    protected $tokenPayload;

    /**
     * @var \SugarDateTime|null - Expire date, or null if there are no limits
     */
    protected $expiryDate;

    /**
     * @var \SugarDateTime - Date of Need To Do PKG installation.
     */
    protected $pluginInstallationDate;

    public const UNCOUNTABLE_LIFETIME_UNIT_KEY = 'No Limits';

    protected $whiteListedLifetimeUnitKeys = array(
        'Days', 'Weeks', 'Months', 'Years', 'No Limits'
    );

    /**
     * StandardValidation constructor.
     * @param $tokenPayload - Decoded License Token
     * @param $pluginInstallationDate - Date of Need To Do PKG installation.
     */
    public function __construct($tokenPayload, $pluginInstallationDate)
    {
        $this->tokenPayload = $tokenPayload;
        $this->pluginInstallationDate = $pluginInstallationDate;
    }

    /**
     * @return array
     */
    public function getTokenPayload()
    {
        return $this->tokenPayload;
    }

    /**
     * No validation for users with Demo token
     * Demo license is visible to everyone
     * @param $user
     */
    public function validateUser($user)
    {

    }

    /**
     * Validates License token
     * @throws InvalidLicenseException
     */
    public function validateLicense()
    {
        $this->checkTokenPayload();
        $this->checkExpired();
    }

    /**
     * @return string
     */
    public function getLicenseType()
    {
        return self::DEMO_LICENSE_TYPE;
    }

    /**
     * Calculates license expiry date
     * @return \SugarDateTime|null
     */
    public function getExpireDate()
    {
        if (isset($this->tokenPayload[self::PK_EXPIRY_DATE])) {
            return $this->getExpiryDateForDateExpirationStrategy();
        } elseif(isset($this->tokenPayload[self::PK_LIFETIME_UNIT])) {
            return $this->getExpiryDateForPeriodExpirationStrategy();
        }
        return null;
    }

    protected function getExpiryDateForDateExpirationStrategy()
    {
        $timeDate = \TimeDate::getInstance();
        $validUntil = $timeDate->fromDbDate($this->tokenPayload[self::PK_EXPIRY_DATE]);
        if ($validUntil) {
            $validUntil->setTime(0, 0, 0);
        }

        return $validUntil;
    }

    protected function getExpiryDateForPeriodExpirationStrategy()
    {
        $lifetimeUnit = $this->getLifetimeUnits();
        $lifetimeDuration = $this->getLifetimeDuration();

        if (empty($lifetimeUnit) || $lifetimeUnit == self::UNCOUNTABLE_LIFETIME_UNIT_KEY) {
            return null;
        }

        $timeDate = \TimeDate::getInstance();
        $pluginInstallationDate = $timeDate->fromDbDate($this->pluginInstallationDate);
        $absoluteLifetimeDuration = abs($lifetimeDuration);
        $pluginInstallationDate->setTime(0, 0, 0);

        if (!empty($absoluteLifetimeDuration)) {
            $intervalSpecification = 'P';
            if ($lifetimeUnit == 'Years') {
                $intervalSpecification .= $absoluteLifetimeDuration . 'Y';
            } elseif ($lifetimeUnit == 'Months') {
                $intervalSpecification .= $absoluteLifetimeDuration . 'M';
            } elseif ($lifetimeUnit == 'Days') {
                $intervalSpecification .= $absoluteLifetimeDuration . 'D';
            } elseif ($lifetimeUnit == 'Weeks') {
                $intervalSpecification .= ($absoluteLifetimeDuration * 7) . 'D';
            }

            try {
                $pluginInstallationDate->add(new \DateInterval($intervalSpecification));
            } catch (\Exception $e) {

            }
        }

        return $pluginInstallationDate;
    }

    /**
     * Demo license is visible to everyone
     * @return null
     */
    public function getCurrentUsers()
    {
        return null;
    }

    /**
     * Demo license is visible to everyone
     */
    public function getUsersLimit()
    {
        return null;
    }

    /**
     * @param $label - Sugar error label key
     * @throws InvalidLicenseException
     */
    private function throwException($label)
    {
        $message = translate($label, 'PX_NeedToDoEvents');
        throw new InvalidLicenseException($message);
    }

    /**
     * Clears static attributes
     */
    public function clearCachedData()
    {

    }

    /**
     * Checks payload
     * @throws InvalidLicenseException
     */
    private function checkTokenPayload()
    {
        if (empty($this->tokenPayload)) {
            $this->throwException('ERROR_LICENSE_EMPTY_TOKEN');
        }

        if (!is_array($this->tokenPayload)) {
            $this->throwException('ERROR_INVALID_LICENSE_TOKEN');
        }

        if (!array_key_exists(self::PK_EXPIRY_DATE, $this->tokenPayload) && !array_key_exists(self::PK_LIFETIME_UNIT, $this->tokenPayload)) {
            $this->throwException('ERROR_INVALID_LICENSE_TOKEN');
        }
    }

    /**
     * Returns License Lifetime units if valid, empty string otherwise
     * @return string
     */
    private function getLifetimeUnits()
    {
        try {
            $this->checkLifetimeUnits();
        } catch(\Exception $e) {
            return '';
        }
        return $this->tokenPayload[self::PK_LIFETIME_UNIT];
    }

    /**
     * Checks License Lifetime units
     * @throws InvalidLicenseException
     */
    private function checkLifetimeUnits()
    {
        $licenseLifetimeUnit = $this->tokenPayload[self::PK_LIFETIME_UNIT];
        if (!is_string($licenseLifetimeUnit) || !in_array($licenseLifetimeUnit, $this->whiteListedLifetimeUnitKeys)) {
            $this->throwException('ERROR_LICENSE_LIFETIME_UNITS_INVALID');
        }
    }

    /**
     * Returns License Duration units if valid, empty string otherwise
     * @return string
     */
    private function getLifetimeDuration()
    {
        try {
            $this->checkLifetimeDuration();
        } catch (\Exception $e) {
            return '';
        }
        return intval($this->tokenPayload[self::PK_LIFETIME_DURATION]);
    }

    /**
     * Checks License Duration units
     * @throws InvalidLicenseException
     */
    private function checkLifetimeDuration()
    {
        $licenseLifetimeUnit = $this->tokenPayload[self::PK_LIFETIME_UNIT];
        $licenseLifetimeDuration = intval($this->tokenPayload[self::PK_LIFETIME_DURATION]);

        if ($licenseLifetimeUnit != self::UNCOUNTABLE_LIFETIME_UNIT_KEY && (!is_int($licenseLifetimeDuration) || $licenseLifetimeDuration < 0)) {
            $this->throwException('ERROR_LICENSE_LIFETIME_DURATION_INVALID');
        }
    }

    /**
     * Checks whether license is expired
     * @throws InvalidLicenseException
     */
    private function checkExpired()
    {
        $now = $this->getNow();
        $validUntil = $this->getExpireDate();

        if (empty($validUntil)) {
            $this->throwException('ERROR_INVALID_LICENSE_TOKEN');
        } else if($now->getTimestamp() > $validUntil->getTimestamp()) {
            $this->throwException('ERROR_LICENSE_TOKEN_EXPIRED');
        }
    }

    private function getNow()
    {
        $timeDate = \TimeDate::getInstance();

        $now = $timeDate->getNow();
        $now->setTime(0, 0, 0);
        return $now;
    }

}
