<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations;

use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Exceptions\InvalidLicenseException;

class StandardValidation implements Validation
{
    /**
     * Standard license type value
     */
    public const STANDARD_LICENSE_TYPE = 'Standard';

    /**
     * Standard Token payload keys
     */
    public const PK_USERS = 'users';
    public const PK_EXPIRY_DATE = 'expiry_date';
    public const PK_SUGAR_INSTANCE_KEY = 'sugar_instance_key';

    /**
     * @var array - Decoded License Token
     */
    protected $tokenPayload;

    /**
     * @var array - required values in token payload
     */
    protected $requiredTokenKeys = array(
        /*'sugar_instance_key',*/ 'users', 'expiry_date'
    );

    /**
     * @var null - Number of users with access to Need To Do Viewer
     */
    protected static $currentUsers = null;

    /**
     * StandardValidation constructor.
     * @param $tokenPayload - Decoded License Token
     */
    public function __construct($tokenPayload)
    {
        $this->tokenPayload = $tokenPayload;
    }

    /**
     * @return array
     */
    public function getTokenPayload()
    {
        return $this->tokenPayload;
    }

    /**
     * Checks access of user to Need To Do Viewer
     * @param \User $user  - target users
     * @throws InvalidLicenseException
     */
    public function validateUser($user)
    {

        if (empty($user->px_ntdv_access_c)) {
            $this->throwException('ERROR_NTDV_INVALID_LICENSE_CURRENT_USER');
        }
    }

    /**
     * Validates License token
     * @throws InvalidLicenseException
     */
    public function validateLicense()
    {
        $this->checkTokenPayload();
        $this->checkInstanceKey();
        $this->checkExpired();
        $this->checkUserLimit();
    }

    /**
     * @return string
     */
    public function getLicenseType()
    {
        return self::STANDARD_LICENSE_TYPE;
    }

    /**
     * Returns license expiry date
     * @return \SugarDateTime
     */
    public function getExpireDate()
    {
        $timeDate = \TimeDate::getInstance();
        $validUntil = $timeDate->fromDbDate($this->tokenPayload[self::PK_EXPIRY_DATE]);
        if($validUntil) {
            $validUntil->setTime(0, 0, 0);
        }

        return $validUntil;
    }

    /**
     * Returns amount of Users with access to Need To Do Viewer
     * @return array|null
     */
    public function getCurrentUsers()
    {
        if (self::$currentUsers === null) {
            require_once 'modules/Users/User.php';
            $db = \DBManagerFactory::getInstance();
            $sql = "SELECT count(id) FROM users LEFT JOIN users_cstm ON users.id = users_cstm.id_c WHERE users_cstm.px_ntdv_access_c = 1 AND ".\User::getLicensedUsersWhere();
            self::$currentUsers = $db->getOne($sql, true);
        }

        return self::$currentUsers;
    }

    /**
     * Returns maximum number of users that can have access to Need To Do Viewer
     * @return mixed
     */
    public function getUsersLimit()
    {
        return intval($this->tokenPayload[self::PK_USERS]);
    }

    /**
     * @param $label - Sugar error label key
     * @throws InvalidLicenseException
     */
    private function throwException($label)
    {
        $message = translate($label, 'PX_NeedToDoEvents');
        throw new InvalidLicenseException($message);
    }

    /**
     * Clears static attributes
     */
    public function clearCachedData()
    {
        self::$currentUsers = null;
    }

    /**
     * Checks payload
     * @throws InvalidLicenseException
     */
    private function checkTokenPayload()
    {
        if (empty($this->tokenPayload)) {
            $this->throwException('ERROR_LICENSE_EMPTY_TOKEN');
        }

        if (!is_array($this->tokenPayload)) {
            $this->throwException('ERROR_INVALID_LICENSE_TOKEN');
        }

        foreach ($this->requiredTokenKeys as $key) {
            if (!array_key_exists($key, $this->tokenPayload)) {
                $this->throwException('ERROR_INVALID_LICENSE_TOKEN');
            }
        }
    }

    /**
     * Checked whether license token was generated to this Sugar Instance
     * @throws InvalidLicenseException
     */
    private function checkInstanceKey()
    {
        global $sugar_config;

        if (!isset($this->tokenPayload[self::PK_SUGAR_INSTANCE_KEY])) {
            return;
        }

        $uniqueKey = $sugar_config['unique_key'];
        $licenseApplicationKey = $this->tokenPayload[self::PK_SUGAR_INSTANCE_KEY];

        if ($uniqueKey != $licenseApplicationKey) {
            $this->throwException('ERROR_LICENSE_INSTANCE_KEY_INVALID');
        }
    }

    /**
     * Checks whether license is expired
     * @throws InvalidLicenseException
     */
    private function checkExpired()
    {
        $now = $this->getNow();
        $validUntil = $this->getExpireDate();

        if ($now->getTimestamp() > $validUntil->getTimestamp()) {
            $this->throwException('ERROR_LICENSE_TOKEN_EXPIRED');
        }
    }

    private function getNow()
    {
        $timeDate = \TimeDate::getInstance();

        $now = $timeDate->getNow();
        $now->setTime(0, 0, 0);
        return $now;
    }

    /**
     * Checks whether maximum user limit is not exceeded
     * @throws InvalidLicenseException
     */
    private function checkUserLimit()
    {
        $currentNumberOfUsers = $this->getCurrentUsers();
        $userLimit = $this->getUsersLimit();

        if ($currentNumberOfUsers > $userLimit) {
            $this->throwException('ERROR_LICENSE_MAX_USERS_LIMIT_EXCEEDED');
        }
    }

}
